import {
  Text,
  Modal,
  ModalContent,
  ModalHeader,
  ModalOverlay,
  ModalProps,
  useDisclosure,
  ModalCloseButton,
  ModalBody,
  VStack,
  HStack,
  FormControl,
  FormLabel,
  Input,
  FormErrorMessage,
  Textarea,
  ModalFooter,
  Button,
} from '@chakra-ui/react'
import { useFormik } from 'formik'
import React from 'react'
import { Pattern } from 'src/models'
import { EditPatternSchema, editPatternSchema } from './schema'
import { UpdatePatternMutation } from '../query-builder/update-pattern-mutation'
import { useQueryClient } from 'react-query'
import { GetPatternQuery, GetPatternsQuery } from '../query-builder'

const FORM_ID = 'edit-pattern-form'

export type EditPatternModalProps = {
  disclosure: ReturnType<typeof useDisclosure>
  pattern: Pattern
} & Partial<ModalProps>

export const EditPatternModal: React.FC<EditPatternModalProps> = ({
  disclosure,
  pattern,
  ...props
}) => {
  const queryClient = useQueryClient()
  const updatePatternMutation = UpdatePatternMutation.useMutation()

  const handleOnSubmit = async (values: EditPatternSchema) => {
    await updatePatternMutation.mutateAsync({
      id: pattern.id,
      name: values.name,
      description: values.description,
    })

    queryClient.invalidateQueries(GetPatternsQuery.getKey({}))
    queryClient.invalidateQueries(GetPatternQuery.getKey({ id: pattern.id }))

    disclosure.onClose()
  }

  const {
    handleSubmit,
    handleReset,
    handleChange,
    setTouched,
    values,
    isSubmitting,
    errors,
    touched,
    isValid,
  } = useFormik({
    enableReinitialize: true,
    initialValues: {
      name: pattern.name,
      description: pattern.description,
    },
    validateOnBlur: true,
    validateOnChange: true,
    validateOnMount: true,
    validationSchema: editPatternSchema,
    onSubmit: handleOnSubmit,
  })

  return (
    <>
      {disclosure.isOpen && (
        <Modal size="xl" {...disclosure} {...props}>
          <ModalOverlay />
          <ModalContent>
            <ModalHeader>
              <Text fontSize="md">Editar patron {pattern.name}</Text>
            </ModalHeader>
            <ModalCloseButton mt="-2" mr="-2" />
            <form id={FORM_ID} onSubmit={handleSubmit} onReset={handleReset}>
              <ModalBody>
                <VStack>
                  <FormControl isInvalid={Boolean(errors.name) && touched.name}>
                    <FormLabel>
                      <HStack>
                        <Text fontSize="sm">Nombre</Text>
                        <Text color="red">*</Text>
                      </HStack>
                    </FormLabel>
                    <Input
                      name="name"
                      autoFocus
                      value={values.name}
                      onChange={(e) => {
                        setTouched({ ...touched, name: true })
                        handleChange(e)
                      }}
                    />
                    <FormErrorMessage>{errors.name}</FormErrorMessage>
                  </FormControl>
                  <FormControl
                    isInvalid={
                      Boolean(errors.description) && touched.description
                    }
                  >
                    <FormLabel>
                      <HStack>
                        <Text fontSize="sm">Descripcion</Text>
                        <Text color="red">*</Text>
                      </HStack>
                    </FormLabel>
                    <Textarea
                      name="description"
                      value={values.description}
                      onChange={(e) => {
                        setTouched({ ...touched, description: true })
                        handleChange(e)
                      }}
                    />
                    <FormErrorMessage>{errors.description}</FormErrorMessage>
                  </FormControl>
                </VStack>
              </ModalBody>
            </form>
            <ModalFooter>
              <HStack>
                <Button variant="outline" onClick={disclosure.onClose}>
                  Cancelar
                </Button>
                <Button
                  form={FORM_ID}
                  variant="solid"
                  colorScheme="blue"
                  type="submit"
                  isLoading={isSubmitting}
                  isDisabled={!isValid}
                >
                  Guardar
                </Button>
              </HStack>
            </ModalFooter>
          </ModalContent>
        </Modal>
      )}
    </>
  )
}
